#include "bsp_wf5803.h"
#include "CONFIG.h"

uint8_t flag;

typedef enum
{
    kPressIn   = 0,
    kPressOut  = 1,
    kPressAtom = 2,
    kPressMaxIndex
} TePressSensorIndex;

static tmosTaskID press_task_id = INVALID_TASK_ID;

#define PRESS_IN_CS_HIGH()   GPIOB_SetBits(GPIO_Pin_9)
#define PRESS_IN_CS_LOW()    GPIOB_ResetBits(GPIO_Pin_9)

#define PRESS_OUT_CS_HIGH()  GPIOB_SetBits(GPIO_Pin_4)
#define PRESS_OUT_CS_LOW()   GPIOB_ResetBits(GPIO_Pin_4)

#define PRESS_ATOM_CS_HIGH() GPIOB_SetBits(GPIO_Pin_17)
#define PRESS_ATOM_CS_LOW()  GPIOB_ResetBits(GPIO_Pin_17)

uint8_t volatile press_done_flag = 0;

uint8_t press_raw_data[kPressMaxIndex][5];

// 1 pa
uint32_t press_value[kPressMaxIndex];

// 0.1 C
int16_t temp_value[kPressMaxIndex];

uint8_t SPI0_SendByte(uint8_t data);
void SPI_CsStart(TePressSensorIndex index);
void SPI_CsStop(TePressSensorIndex index);

/* Variable to store the device address */
static uint8_t dev_in_addr;
static uint8_t dev_out_addr;

uint8_t Bmp_ReadData(uint8_t *reg_data, uint32_t len)
{
    while (len--)
    {
        *reg_data = SPI0_SendByte(0x00);
        reg_data++;
    }
    return BMP3_INTF_RET_SUCCESS;
}

BMP3_INTF_RET_TYPE Bmp_WriteData(const uint8_t *reg_data, uint32_t len)
{
    uint8_t i = 0;
    for (i = 0; i < len; i++)
    {
        SPI0_SendByte(reg_data[i]);
    }
    return BMP3_INTF_RET_SUCCESS;
}

BMP3_INTF_RET_TYPE BMP390_IN_SPI_Read(uint8_t reg_addr, uint8_t *reg_data, uint32_t len, void *intf_ptr)
{
    BMP3_INTF_RET_TYPE rslt = 0;

    uint8_t reg_spi[1] = {(reg_addr & 0x7F) | 0x80};
    SPI_CsStart(kPressIn);     // Ƭѡ
    Bmp_WriteData(reg_spi, 1); // дֽ
    rslt = Bmp_ReadData(reg_data, len);
    SPI_CsStop(kPressIn);
    return rslt;
}

/*!
 * SPI write function map to COINES platform
 */
BMP3_INTF_RET_TYPE BMP390_IN_SPI_Write(uint8_t reg_addr, const uint8_t *reg_data, uint32_t len, void *intf_ptr)
{
    uint8_t reg_spi[1]      = {reg_addr & 0x7f};
    BMP3_INTF_RET_TYPE rslt = 0;

    SPI_CsStart(kPressIn);
    Bmp_WriteData(reg_spi, 1);
    rslt = Bmp_WriteData(reg_data, len);
    SPI_CsStop(kPressIn);
    //    printf("BMP390_OUT_SPI_Write: %d" , rslt);

    return rslt;
}

/*!
 * SPI read function map to COINES platform
 */
BMP3_INTF_RET_TYPE BMP390_OUT_SPI_Read(uint8_t reg_addr, uint8_t *reg_data, uint32_t len, void *intf_ptr)
{
    BMP3_INTF_RET_TYPE rslt = 0;

    uint8_t reg_spi[1] = {(reg_addr & 0x7F) | 0x80};
    SPI_CsStart(kPressOut);    // Ƭѡ
    Bmp_WriteData(reg_spi, 1); // дֽ
    rslt = Bmp_ReadData(reg_data, len);
    SPI_CsStop(kPressOut);
    return rslt;
}

/*!
 * SPI write function map to COINES platform
 */
BMP3_INTF_RET_TYPE BMP390_OUT_SPI_Write(uint8_t reg_addr, const uint8_t *reg_data, uint32_t len, void *intf_ptr)
{
    uint8_t reg_spi[1]      = {reg_addr & 0x7f};
    BMP3_INTF_RET_TYPE rslt = 0;

    SPI_CsStart(kPressOut);
    Bmp_WriteData(reg_spi, 1);
    rslt = Bmp_WriteData(reg_data, len);
    SPI_CsStop(kPressOut);
    //    printf("BMP390_OUT_SPI_Write: %d" , rslt);

    return rslt;
}

void bmp3_delay_us(uint32_t period, void *intf_ptr)
{
    DelayUs(period);
}

void bmp3_check_rslt(const char api_name[], int8_t rslt)
{
    switch (rslt)
    {
        case BMP3_OK:

            /* Do nothing */
            break;
        case BMP3_E_NULL_PTR:
            printf("API [%s] Error [%d] : Null pointer\r\n", api_name, rslt);
            break;
        case BMP3_E_COMM_FAIL:
            printf("API [%s] Error [%d] : Communication failure\r\n", api_name, rslt);
            break;
        case BMP3_E_INVALID_LEN:
            printf("API [%s] Error [%d] : Incorrect length parameter\r\n", api_name, rslt);
            break;
        case BMP3_E_DEV_NOT_FOUND:
            printf("API [%s] Error [%d] : Device not found\r\n", api_name, rslt);
            break;
        case BMP3_E_CONFIGURATION_ERR:
            printf("API [%s] Error [%d] : Configuration Error\r\n", api_name, rslt);
            break;
        case BMP3_W_SENSOR_NOT_ENABLED:
            printf("API [%s] Error [%d] : Warning when Sensor not enabled\r\n", api_name, rslt);
            break;
        case BMP3_W_INVALID_FIFO_REQ_FRAME_CNT:
            printf("API [%s] Error [%d] : Warning when Fifo watermark level is not in limit\r\n", api_name, rslt);
            break;
        default:
            printf("API [%s] Error [%d] : Unknown error code\r\n", api_name, rslt);
            break;
    }
}

BMP3_INTF_RET_TYPE BMP390_IN_InterfaceInit(struct bmp3_dev *bmp3, uint8_t intf)
{
    int8_t rslt = BMP3_OK;

    /* Bus configuration : SPI */
    if (intf == BMP3_SPI_INTF)
    {
        printf("SPI Interface\n");
        bmp3->read  = BMP390_IN_SPI_Read;
        bmp3->write = BMP390_IN_SPI_Write;
        bmp3->intf  = BMP3_SPI_INTF;
        printf("spi init ok\r\n");
    }

    DelayMs(100);
    bmp3->delay_us = bmp3_delay_us;
    bmp3->intf_ptr = &dev_in_addr;

    return rslt;
}

BMP3_INTF_RET_TYPE BMP390_OUT_InterfaceInit(struct bmp3_dev *bmp3, uint8_t intf)
{
    int8_t rslt = BMP3_OK;

    /* Bus configuration : SPI */
    if (intf == BMP3_SPI_INTF)
    {
        printf("SPI Interface\n");
        bmp3->read  = BMP390_OUT_SPI_Read;
        bmp3->write = BMP390_OUT_SPI_Write;
        bmp3->intf  = BMP3_SPI_INTF;
        printf("spi init ok\r\n");
    }

    DelayMs(100);
    bmp3->delay_us = bmp3_delay_us;
    bmp3->intf_ptr = &dev_out_addr;

    return rslt;
}

void SPI_CsStart(TePressSensorIndex index)
{
    switch (index)
    {
        case kPressIn:
            PRESS_IN_CS_LOW();
            break;
        case kPressOut:
            PRESS_OUT_CS_LOW();
            break;
        case kPressAtom:
            PRESS_ATOM_CS_LOW();
            break;

        default:
            break;
    }
}

void SPI_CsStop(TePressSensorIndex index)
{
    switch (index)
    {
        case kPressIn:
            PRESS_IN_CS_HIGH();
            break;
        case kPressOut:
            PRESS_OUT_CS_HIGH();
            break;
        case kPressAtom:
            PRESS_ATOM_CS_HIGH();
            break;

        default:
            break;
    }
}

uint8_t SPI0_SendByte(uint8_t data)
{
    R8_SPI0_BUFFER = data;
    while (!(R8_SPI0_INT_FLAG & RB_SPI_FREE));
    return (R8_SPI0_BUFFER);
}

void WF5803_WriteReg(uint8_t Address, uint8_t value, TePressSensorIndex index)
{
    SPI_CsStart(index);
    SPI0_SendByte(0x00);
    SPI0_SendByte(Address);
    SPI0_SendByte(value);
    SPI_CsStop(index);
}

uint8_t WF5803_ReadReg(uint8_t addr, TePressSensorIndex index)
{
    uint8_t value;
    SPI_CsStart(index);
    SPI0_SendByte(0x80);
    SPI0_SendByte(addr);
    value = SPI0_SendByte(0xFF);
    SPI_CsStop(index);
    return value;
}

void PRESS_IO_SPI_Init(void)
{
    /**
     * CSB: PB17
     * SCL: PA13
     * SDA: PA14
     * SDO: PA15
     */
    // SDA: MOSI
    // SDO: MISO

    // CSB1: PA3
    // GPIOA_SetBits(GPIO_Pin_3);
    // GPIOA_ModeCfg(GPIO_Pin_3, GPIO_ModeOut_PP_5mA);

    // CSB2: PB9
    GPIOB_SetBits(GPIO_Pin_9);
    GPIOB_ModeCfg(GPIO_Pin_9, GPIO_ModeOut_PP_5mA);

    // CSB3: PB4
    GPIOB_SetBits(GPIO_Pin_4);
    GPIOB_ModeCfg(GPIO_Pin_4, GPIO_ModeOut_PP_5mA);

    // CSB4: PB17
    GPIOB_SetBits(GPIO_Pin_17);
    GPIOB_ModeCfg(GPIO_Pin_17, GPIO_ModeOut_PP_5mA);

    SPI_CsStop(kPressIn);
    SPI_CsStop(kPressOut);
    SPI_CsStop(kPressAtom);

    // spiʼģʽ0
    GPIOA_ModeCfg(GPIO_Pin_13 | GPIO_Pin_14, GPIO_ModeOut_PP_5mA);
    GPIOA_ModeCfg(GPIO_Pin_15, GPIO_ModeIN_PU);

    SPI0_MasterDefInit();
}

void WF5803_Init(void)
{
    PRESS_IO_SPI_Init();
    WF5803_WriteReg(0x00, 0x81, kPressIn);   // spiΪģʽ
    WF5803_WriteReg(0x00, 0x81, kPressOut);  // spiΪģʽ
    WF5803_WriteReg(0x00, 0x81, kPressAtom); // spiΪģʽ
}

void PRESS_LowerIO_Init(void)
{
    // WF5803ĬϹʱIOǸߵƽ
    // SPI
    GPIOA_SetBits(GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15);
    GPIOA_ModeCfg(GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15, GPIO_ModeIN_PU);

    // CSB1: PA3
    // GPIOA_SetBits(GPIO_Pin_3);
    // GPIOA_ModeCfg(GPIO_Pin_3, GPIO_ModeIN_PU);
    // CSB1: PA3
    // GPIOA_SetBits(GPIO_Pin_3);
    // GPIOA_ModeCfg(GPIO_Pin_3, GPIO_ModeOut_PP_5mA);

    // CSB2: PB9
    GPIOB_SetBits(GPIO_Pin_9);
    GPIOB_ModeCfg(GPIO_Pin_9, GPIO_ModeIN_PU);

    // CSB3: PB4
    GPIOB_SetBits(GPIO_Pin_4);
    GPIOB_ModeCfg(GPIO_Pin_4, GPIO_ModeIN_PU);

    // CSB4: PB17
    GPIOB_SetBits(GPIO_Pin_17);
    GPIOB_ModeCfg(GPIO_Pin_17, GPIO_ModeIN_PU);
}

void Lower_IO_Deinit(void)
{
    // LED
    GPIOA_ResetBits(GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_12);
    GPIOA_ModeCfg(GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_12, GPIO_ModeIN_PD);

    // KEY
    GPIOA_ResetBits(GPIO_Pin_7);
    GPIOA_ModeCfg(GPIO_Pin_7, GPIO_ModeIN_PU);

    // RESET KEY
    GPIOB_ResetBits(GPIO_Pin_7);
    GPIOB_ModeCfg(GPIO_Pin_7, GPIO_ModeIN_PU);

    // motor
    GPIOB_ResetBits(GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2);
    GPIOB_ModeCfg(GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2, GPIO_ModeIN_PD);

    // COIL_ADC
    GPIOA_ResetBits(GPIO_Pin_6);
    GPIOA_ModeCfg(GPIO_Pin_6, GPIO_ModeIN_PD);

    // ADC_CTRL,ADC_VBAT
    GPIOA_ResetBits(GPIO_Pin_0 | GPIO_Pin_1);
    GPIOA_ModeCfg(GPIO_Pin_0 | GPIO_Pin_1, GPIO_ModeIN_PD);
}

void PRESS_LowPower(void)
{
    Lower_IO_Deinit();
    if (press_done_flag == 1)
    {
        PRESS_LowerIO_Init();
    }
}

uint8_t GetSensorData(TePressSensorIndex index)
{
    memset(press_raw_data[index], 0, 5);

    if (WF5803_ReadReg(0x02, index) != 0x01)
    {
        PRINT("Status = %02x\r\n", WF5803_ReadReg(0x02, index));
        return 1;
    }

    // ѹ
    // ¶
    for (uint8_t i = 0; i < 5; i++)
    {
        press_raw_data[index][i] = WF5803_ReadReg(0x06 + i, index);
    }

    return 0;
}

__HIGH_CODE
__attribute__((noinline)) void SensorData_Process(TePressSensorIndex index)
{
    long reading = 0;
    float fDat   = 0;
    float press  = 0;
    float temp   = 0;

    GetSensorData(index);

    reading = press_raw_data[index][0];
    reading = reading << 8;
    reading |= press_raw_data[index][1];
    reading = reading << 8;
    reading |= press_raw_data[index][2];
    if (reading >= 8388608)
    {
        fDat = (int32_t)(reading - 16777216) / 8388608.0f;
    }
    else
    {
        fDat = reading / 8388608.0f;
    }

    press              = fDat * 125 + 17.5;        // WF5803_1BAR    ʹ10mʹ
    press_value[index] = (uint32_t)(press * 1000); // pa

    reading = press_raw_data[index][3];
    reading = reading << 8;
    reading |= press_raw_data[index][4];
    if (reading > 32768)
    {
        temp              = (reading - 65844) / 256.0f;
        temp_value[index] = (int16_t)(temp * 10); // .1 C
    }
    else
    {
        temp              = (reading - 308) / 256.0f;
        temp_value[index] = (int16_t)(temp * 10); // .1 C
    }

    // PRINT("P[%d] = %d.%d pa\r\n", index, (int)(press * 1000), ((int)(press * 10000000.0f) % 10000));
    // PRINT("T[%d] = %d.%d \r\n", index, (int)temp, ((int)(temp * 100.0f) % 100));
}

int8_t ret   = 0;
uint8_t loop = 0;
struct bmp3_dev DevIn;
struct bmp3_dev DevOut;
uint16_t settings_sel;
struct bmp3_data data         = {0};
struct bmp3_settings settings = {0};
struct bmp3_status status     = {{0}};

__HIGH_CODE
__attribute__((noinline))
uint16_t
WF5803_ProcessEvent(uint8_t task_id, uint16_t events)
{
    if (events & WF5803_EVT_START)
    {
        press_done_flag = 0;

#if 0
        WF5803_Init();

        WF5803_WriteReg(0x30, 0x0A, kPressIn);
        WF5803_WriteReg(0x30, 0x0A, kPressOut);
        WF5803_WriteReg(0x30, 0x0A, kPressAtom);
#endif
        PRESS_IO_SPI_Init();
        DelayMs(2);

        settings.op_mode = BMP3_MODE_FORCED;
        ret              = bmp3_set_op_mode(&settings, &DevIn);
        bmp3_check_rslt("bmp3_set_op_mode", ret);

        settings.op_mode = BMP3_MODE_FORCED;
        ret              = bmp3_set_op_mode(&settings, &DevOut);
        bmp3_check_rslt("bmp3_set_op_mode", ret);

        // tmos_start_task(press_task_id, WF5803_EVT_READ, MS1_TO_SYSTEM_TIME(5));
        return (events ^ WF5803_EVT_START);
    }
    else if (events & WF5803_EVT_READ)
    {
#if 0
        SensorData_Process(kPressIn);
        SensorData_Process(kPressOut);
        SensorData_Process(kPressAtom);

        press_value[kPressIn];
        press_value[kPressOut];
        press_value[kPressAtom];

        PRINT("%d %d %d pa\r\n", press_value[kPressIn], press_value[kPressOut], press_value[kPressAtom]);
        PRINT("%d %d %d C\r\n", temp_value[kPressIn], temp_value[kPressOut], temp_value[kPressAtom]);

        if (press_value[kPressIn] > press_value[kPressOut])
        {
            PRINT("> %d\r\n", press_value[kPressIn] - press_value[kPressOut]);
        }
        else
        {
            PRINT("< %d\r\n", press_value[kPressOut] - press_value[kPressIn]);
        }

#endif

#if 0
         PRESS_IO_SPI_Init();

                // IN
                ret = bmp3_get_status(&status, &DevIn);
                bmp3_check_rslt("bmp3_get_status", ret);

                /* Read temperature and pressure data iteratively based on data ready interrupt */
                if ((ret == BMP3_OK) && (status.intr.drdy == BMP3_ENABLE))
                {
                    /*
                     * First parameter indicates the type of data to be read
                     * BMP3_PRESS_TEMP : To read pressure and temperature data
                     * BMP3_TEMP       : To read only temperature data
                     * BMP3_PRESS      : To read only pressure data
                     */
                    ret = bmp3_get_sensor_data(BMP3_PRESS_TEMP, &data, &DevIn);
                    bmp3_check_rslt("bmp3_get_sensor_data", ret);

                    /* NOTE : Read status register again to clear data ready interrupt status */
                    ret = bmp3_get_status(&status, &DevIn);
                    bmp3_check_rslt("bmp3_get_status", ret);

         #ifdef BMP3_FLOAT_COMPENSATION
                    printf("IN[%d]  T: %.2f deg C, P: %.2f Pa\n", loop, (data.temperature), (data.pressure));
         #else
                    printf("IN[%d]  T: %ld deg C, P: %lu Pa\n", loop, (long int)(int32_t)(data.temperature / 100),
                           (long unsigned int)(uint32_t)(data.pressure / 100));
         #endif
                }

                // OUT
                ret = bmp3_get_status(&status, &DevOut);
                bmp3_check_rslt("bmp3_get_status", ret);

                /* Read temperature and pressure data iteratively based on data ready interrupt */
                if ((ret == BMP3_OK) && (status.intr.drdy == BMP3_ENABLE))
                {
                    /*
                     * First parameter indicates the type of data to be read
                     * BMP3_PRESS_TEMP : To read pressure and temperature data
                     * BMP3_TEMP       : To read only temperature data
                     * BMP3_PRESS      : To read only pressure data
                     */
                    ret = bmp3_get_sensor_data(BMP3_PRESS_TEMP, &data, &DevOut);
                    bmp3_check_rslt("bmp3_get_sensor_data", ret);

                    /* NOTE : Read status register again to clear data ready interrupt status */
                    ret = bmp3_get_status(&status, &DevOut);
                    bmp3_check_rslt("bmp3_get_status", ret);

         #ifdef BMP3_FLOAT_COMPENSATION
                    printf("OUT[%d]  T: %.2f deg C, P: %.2f Pa\n", loop, (data.temperature), (data.pressure));
         #else
                    printf("OUT[%d]  T: %ld deg C, P: %lu Pa\n", loop, (long int)(int32_t)(data.temperature / 100),
                           (long unsigned int)(uint32_t)(data.pressure / 100));
         #endif
                    loop = loop + 1;
                }
        tmos_start_task(press_task_id, WF5803_EVT_START, MS1_TO_SYSTEM_TIME(2000));
#endif
       if(flag == 1)
       {
          PRESS_IO_SPI_Init();
          ret = bmp3_get_status(&status, &DevIn); // жΪģʽҪȡint_status.drdyλж״̬־
          bmp3_check_rslt("bmp3_get_status", ret);

          if (status.intr.drdy == BMP3_ENABLE)
          {
              /*
               * First parameter indicates the type of data to be read
               * BMP3_PRESS_TEMP : To read pressure and temperature data
               * BMP3_TEMP       : To read only temperature data
               * BMP3_PRESS      : To read only pressure data
               */
              ret = bmp3_get_sensor_data(BMP3_PRESS_TEMP, &data, &DevIn);
              bmp3_check_rslt("bmp3_get_sensor_data", ret);

              /* NOTE : Read status register again to clear data ready interrupt status */
              ret = bmp3_get_status(&status, &DevIn);
              bmp3_check_rslt("bmp3_get_status", ret);

              printf("IN[%d]  T: %ld deg C, P: %lu Pa\n", loop, (long int)(int32_t)(data.temperature / 100),
                     (long unsigned int)(uint32_t)(data.pressure / 100));
          }
       }
       if(flag == 2)
       {
            PRESS_IO_SPI_Init();

            ret = bmp3_get_status(&status, &DevOut); // жΪģʽҪȡint_status.drdyλж״̬־
            bmp3_check_rslt("bmp3_get_status", ret);

            if (status.intr.drdy == BMP3_ENABLE)
            {
                /*
                 * First parameter indicates the type of data to be read
                 * BMP3_PRESS_TEMP : To read pressure and temperature data
                 * BMP3_TEMP       : To read only temperature data
                 * BMP3_PRESS      : To read only pressure data
                 */
                ret = bmp3_get_sensor_data(BMP3_PRESS_TEMP, &data, &DevOut);
                bmp3_check_rslt("bmp3_get_sensor_data", ret);

                /* NOTE : Read status register again to clear data ready interrupt status */
                ret = bmp3_get_status(&status, &DevOut);
                bmp3_check_rslt("bmp3_get_status", ret);

                printf("OUT[%d]  T: %ld deg C, P: %lu Pa\n", loop, (long int)(int32_t)(data.temperature / 100),
                                          (long unsigned int)(uint32_t)(data.pressure / 100));
            }
       }
        loop = loop + 1;
        press_done_flag = 1;
        tmos_start_task(press_task_id, WF5803_EVT_START, MS1_TO_SYSTEM_TIME(1000));
        return (events ^ WF5803_EVT_READ);
    }
    return 0;
}

void BSP_PRESS_Init(void)
{
    PRESS_IO_SPI_Init();

    // жŵ PB2 PB3
    GPIOB_ModeCfg(GPIO_Pin_2, GPIO_ModeIN_PD);
    GPIOB_ITModeCfg(GPIO_Pin_2, GPIO_ITMode_RiseEdge);

    GPIOB_ModeCfg(GPIO_Pin_3, GPIO_ModeIN_PD);
    GPIOB_ITModeCfg(GPIO_Pin_3, GPIO_ITMode_RiseEdge);

    PWR_PeriphWakeUpCfg(ENABLE, RB_GPIO_WAKE_MODE | RB_SLP_GPIO_WAKE, Long_Delay);
    PFIC_EnableIRQ(GPIO_B_IRQn);

    // IN
    ret = BMP390_IN_InterfaceInit(&DevIn, BMP3_SPI_INTF);
    bmp3_check_rslt("BMP390_OUT_InterfaceInit", ret);

    ret = bmp3_init(&DevIn);
    bmp3_check_rslt("bmp3_init", ret);
    settings.int_settings.drdy_en     = BMP3_ENABLE;
    settings.int_settings.latch       = BMP3_INT_PIN_LATCH;
    settings.int_settings.level       = BMP3_INT_PIN_ACTIVE_HIGH;
    settings.int_settings.output_mode = BMP3_INT_PIN_PUSH_PULL;

    settings.press_en = BMP3_ENABLE;
    settings.temp_en  = BMP3_ENABLE;

    settings.odr_filter.press_os   = BMP3_OVERSAMPLING_2X;
    settings.odr_filter.temp_os    = BMP3_OVERSAMPLING_2X;
    settings.odr_filter.odr        = BMP3_ODR_1_5_HZ;
    settings.odr_filter.iir_filter = BMP3_IIR_FILTER_COEFF_3;

    settings_sel = BMP3_SEL_PRESS_EN | BMP3_SEL_TEMP_EN | BMP3_SEL_PRESS_OS | BMP3_SEL_TEMP_OS | BMP3_SEL_ODR | BMP3_SEL_DRDY_EN | BMP3_SEL_IIR_FILTER | BMP3_SEL_OUTPUT_MODE | BMP3_SEL_LEVEL | BMP3_SEL_LATCH;

    ret = bmp3_set_sensor_settings(settings_sel, &settings, &DevIn);
    bmp3_check_rslt("bmp3_set_sensor_settings", ret);

    //    settings.op_mode = BMP3_MODE_NORMAL;
    //    ret              = bmp3_set_op_mode(&settings, &DevIn);
    //    bmp3_check_rslt("bmp3_set_op_mode", ret);

    // OUT
    ret = BMP390_OUT_InterfaceInit(&DevOut, BMP3_SPI_INTF);
    bmp3_check_rslt("BMP390_OUT_InterfaceInit", ret);

    ret = bmp3_init(&DevOut);
    bmp3_check_rslt("bmp3_init", ret);
    settings.int_settings.drdy_en     = BMP3_ENABLE;
    settings.int_settings.latch       = BMP3_INT_PIN_LATCH;
    settings.int_settings.level       = BMP3_INT_PIN_ACTIVE_HIGH;
    settings.int_settings.output_mode = BMP3_INT_PIN_PUSH_PULL;
    settings.press_en                 = BMP3_ENABLE;
    settings.temp_en                  = BMP3_ENABLE;

    settings.odr_filter.press_os   = BMP3_OVERSAMPLING_2X;
    settings.odr_filter.temp_os    = BMP3_OVERSAMPLING_2X;
    settings.odr_filter.odr        = BMP3_ODR_1_5_HZ;
    settings.odr_filter.iir_filter = BMP3_IIR_FILTER_COEFF_3;

    settings_sel = BMP3_SEL_PRESS_EN | BMP3_SEL_TEMP_EN | BMP3_SEL_PRESS_OS | BMP3_SEL_TEMP_OS | BMP3_SEL_ODR | BMP3_SEL_DRDY_EN | BMP3_SEL_IIR_FILTER | BMP3_SEL_OUTPUT_MODE | BMP3_SEL_LEVEL | BMP3_SEL_LATCH;

    ret = bmp3_set_sensor_settings(settings_sel, &settings, &DevOut);
    bmp3_check_rslt("bmp3_set_sensor_settings", ret);

    //    settings.op_mode = BMP3_MODE_NORMAL;
    //    ret              = bmp3_set_op_mode(&settings, &DevOut);
    //    bmp3_check_rslt("bmp3_set_op_mode", ret);

    press_task_id = TMOS_ProcessEventRegister(WF5803_ProcessEvent);
    tmos_set_event(press_task_id, WF5803_EVT_START);
}

#if 0
volatile uint8_t flag = 0;
// __INTERRUPT
// __HIGH_CODE
void BMP390_IN_IRQ(void)
{
    if (flag)
    {
        flag = 0;
        ret  = bmp3_get_status(&status, &DevIn); // жΪģʽҪȡint_status.drdyλж״̬־
        bmp3_check_rslt("bmp3_get_status", ret);

        if (status.intr.drdy == BMP3_ENABLE)
        {
            printf("interrupt\r\n");
            /*
             * First parameter indicates the type of data to be read
             * BMP3_PRESS_TEMP : To read pressure and temperature data
             * BMP3_TEMP       : To read only temperature data
             * BMP3_PRESS      : To read only pressure data
             */
            ret = bmp3_get_sensor_data(BMP3_PRESS_TEMP, &data, &DevIn);
            bmp3_check_rslt("bmp3_get_sensor_data", ret);

            /* NOTE : Read status register again to clear data ready interrupt status */
            ret = bmp3_get_status(&status, &DevIn);
            bmp3_check_rslt("bmp3_get_status", ret);

            printf("IN[%d]  T: %ld deg C, P: %lu Pa\n", loop, (long int)(int32_t)(data.temperature / 100),
                   (long unsigned int)(uint32_t)(data.pressure / 100));
            loop = loop + 1;
        }
    }
}
#endif

__INTERRUPT
__HIGH_CODE
void GPIOB_IRQHandler(void)
{
    if (R16_PB_INT_IF & GPIO_Pin_2)
    {
        R16_PB_INT_IF = GPIO_Pin_2;
        flag = 1;
        tmos_set_event(press_task_id, WF5803_EVT_READ);
        printf("interrupt1\r\n");
    }
    if (R16_PB_INT_IF & GPIO_Pin_3)
    {
        R16_PB_INT_IF = GPIO_Pin_3;
        flag = 2;
        tmos_set_event(press_task_id, WF5803_EVT_READ);
        printf("interrupt2\r\n");
    }
}
